//
//  NisusSpatialArray.h
//  NisusCore
//
//  Created by Martin on 2019.01.28.
//  Copyright © 2019 Nisus. All rights reserved.
//
//	This is an array that keeps its objects sorted by a single spatial dimension, eg: Y coordinate.
//	The objects in the array are allowed not only a coordinate, but also a length.
//	In this way it's easy to enumerate all objects in the array that intersect some region.

#import <Foundation/Foundation.h>
#import <NisusCore/NisusSpatialEdge.h>



@interface NisusSpatialArray : NSObject

- (void) addObject:(id)object withEdge:(NisusSpatialEdge)edge ;
- (void) setEdge:(NisusSpatialEdge)edge forObject:(id)object ;
- (void) removeObject:(id)object ;
- (void) removeAllObjects ;

// simple info
- (NSUInteger) count ;
- (NisusSpatialEdge) spatialBounds ;

- (BOOL) containsObject:(id)object ;
- (NisusSpatialEdge) edgeForObject:(id)object ;

// queries
- (void) enumerateObjects:(void(^)(id object, NisusSpatialEdge edge, BOOL* stop))visitor ;
- (void) enumerateObjectsInEdge:(NisusSpatialEdge)testEdge using:(void(NS_NOESCAPE^)(id object, NisusSpatialEdge edge, BOOL* stop))visitor ;

- (NSArray*) objectsInEdge:(NisusSpatialEdge)testEdge ;
- (NSArray*) objectsInEdge:(NisusSpatialEdge)testEdge passingTest:(BOOL(NS_NOESCAPE^)(id object, NisusSpatialEdge edge, BOOL* stop))test ;

@end

