//
//  NisusStringReplacer.h
//  NisusCore
//
//  Created by Martin on 2018.11.16.
//  Copyright © 2018 Nisus. All rights reserved.
//
//	This class helps with optimized string filtering / replacements:
//	1. Making a copy of the string is deferred unless changes are actually required.
//	2. Replacements in large strings may use a character buffer.

#import <Foundation/Foundation.h>

// A matcher should return the range to replace, or NSNotFound range if no matches are made.
// If a valid range is returned the replacement string pointer must also be set.
typedef NSRange (^NisusStringReplacerMatcher)(NSString* searchString, NSRange searchRange, NSString** replacementStrPtr);



@interface NisusStringReplacer : NSObject

+ (NSString*) filterString:(NSString*)source using:(void(^)(NisusStringReplacer* replacer))accessor ;

+ (instancetype) replacerWithString:(NSString*)source ;
- (instancetype) initWithString:(NSString*)source ;
- (instancetype) initWithMutableString:(NSMutableString*)source ;

// results
- (NSString*) currentString ;
- (NSRange) currentStringRange ;
- (NSString*) finalizedString ;

- (BOOL) isMutated ;
- (void) accessMutableString:(void(^)(NSMutableString* mstring))accessor ;

// base replacement operation
- (NSUInteger) replaceAllUsing:(NisusStringReplacerMatcher)matcher ;
- (NSUInteger) replaceAllInRange:(NSRange)fullRange using:(NisusStringReplacerMatcher)matcher ;

+ (NSUInteger) replaceAllInString:(NSMutableString*)mstring range:(NSRange)searchRange using:(NisusStringReplacerMatcher)matcher ;
#ifdef NISUS_UNIT_TEST
+ (NSMutableString*) newStringByReplacingAllBuffered:(NisusStringReplacerMatcher)matcher forString:(NSString*)origString range:(NSRange)requestRange getMatchCount:(NSUInteger*)replaceCountPtr ;
#endif

// convenience
- (NSUInteger) replaceAllOccurrencesOfString:(NSString*)findString withString:(NSString*)replaceString ;
- (NSUInteger) replaceAllOccurrencesOfString:(NSString*)findString range:(NSRange)fullRange withString:(NSString*)replaceString ;
- (NSUInteger) replaceAllOccurrencesOfString:(NSString*)findString options:(NSStringCompareOptions)ops range:(NSRange)fullRange withString:(NSString*)replaceString ;

- (NSUInteger) replaceAllCharactersInSet:(NSCharacterSet*)findSet withString:(NSString*)replaceString ;
- (NSUInteger) replaceAllCharactersInSet:(NSCharacterSet*)findSet range:(NSRange)fullRange withString:(NSString*)replaceString ;
- (NSUInteger) replaceAllCharactersInSet:(NSCharacterSet*)findSet coalesced:(BOOL)isCoalesce range:(NSRange)fullRange withString:(NSString*)replaceString ;

@end

